@@ -1,6 +1,6 @@
 /*
  * POSIX library for Lua 5.1, 5.2 & 5.3.
- * Copyright (C) 2013-2018 Gary V. Vaughan
+ * Copyright (C) 2013-2019 Gary V. Vaughan
  * Copyright (C) 2010-2013 Reuben Thomas <rrt@sc3d.org>
  * Copyright (C) 2008-2010 Natanael Copa <natanael.copa@gmail.com>
  * Clean up and bug fixes by Leo Razoumov <slonik.az@gmail.com> 2006-10-11
@@ -36,6 +36,7 @@
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 #include <netinet/udp.h>
+#include <sys/time.h>
 #include <sys/un.h>
 
 
@@ -90,6 +91,8 @@ pushsockaddrinfo(lua_State *L, int family, struct sockaddr *sa)
 			inet_ntop(family, &sa6->sin6_addr, addr, sizeof addr);
 			pushintegerfield("port", ntohs(sa6->sin6_port));
 			pushstringfield("addr", addr);
+			pushintegerfield("flowinfo", ntohl(sa6->sin6_flowinfo));
+			pushintegerfield("scope_id", sa6->sin6_scope_id);
 			break;
 		}
 		case AF_UNIX:
@@ -172,12 +175,18 @@ Psocketpair(lua_State *L)
 }
 
 
-static const char *Safinet_fields[] = { "family", "port", "addr",
+static const char *Safinet_fields[] = { /* AF_INET6 only */
+					"flowinfo", "scope_id",
+					/* AF_INET4 and AF_INET6 shared */
+					"family", "port", "addr",
 					/* Also allow getaddrinfo result tables */
 					"socktype", "canonname", "protocol" };
 static const char *Safunix_fields[] = { "family", "path" };
 static const char *Safnetlink_fields[] = { "family", "pid", "groups" };
 
+#define Safinet4_fields (Safinet_fields + 2)
+#define Safinet6_fields Safinet_fields
+
 
 /* Populate a sockaddr_storage with the info from the given lua table */
 static int
@@ -197,8 +206,9 @@ sockaddr_from_lua(lua_State *L, int index, struct sockaddr_storage *sa, socklen_
 			struct sockaddr_in *sa4 = (struct sockaddr_in *)sa;
 			int port		= checkintfield(L, index, "port");
 			const char *addr	= checkstringfield(L, index, "addr");
+			int nfields = (sizeof(Safinet_fields)/sizeof(*Safinet_fields))-2;
 
-			checkfieldnames (L, index, Safinet_fields);
+			(checkfieldnames)(L, index, nfields, Safinet4_fields);
 
 			if (inet_pton(AF_INET, addr, &sa4->sin_addr) == 1)
 			{
@@ -214,13 +224,18 @@ sockaddr_from_lua(lua_State *L, int index, struct sockaddr_storage *sa, socklen_
 			struct sockaddr_in6 *sa6= (struct sockaddr_in6 *)sa;
 			int port		= checkintfield(L, index, "port");
 			const char *addr	= checkstringfield(L, index, "addr");
+			/* For compatibility, the next two fields are optional */
+			uint32_t flowinfo	= optintfield(L, index, "flowinfo", 0);
+			uint32_t scope_id	= optintfield(L, index, "scope_id", 0);
 
-			checkfieldnames (L, index, Safinet_fields);
+			checkfieldnames (L, index, Safinet6_fields);
 
 			if (inet_pton(AF_INET6, addr, &sa6->sin6_addr) == 1)
 			{
 				sa6->sin6_family= family;
 				sa6->sin6_port	= htons(port);
+				sa6->sin6_flowinfo	= htonl(flowinfo);
+				sa6->sin6_scope_id	= scope_id;
 				*addrlen	= sizeof(*sa6);
 				r		= 0;
 			}
@@ -275,7 +290,7 @@ static const char *Sai_fields[] = { "family", "socktype", "protocol", "flags" };
 /***
 Network address and service translation.
 @function getaddrinfo
-@string host name of a host
+@string host name of a host. For IPv6, link-local addresses like 'ff02::1%eth0' are supported.
 @string service name of service
 @tparam[opt] PosixAddrInfo hints table
 @treturn[1] list of @{sockaddr} tables
@@ -386,7 +401,7 @@ Pconnect(lua_State *L)
 	int fd = checkint(L, 1);
 	checknargs (L, 2);
 	if (sockaddr_from_lua(L, 2, &sa, &salen) != 0)
-		return pusherror(L, "not a valid IPv4 dotted-decimal or IPv6 address string");
+		return pusherror(L, "not a valid IPv4 or IPv6 address argument");
 
 	return pushresult(L, connect(fd, (struct sockaddr *)&sa, salen), "connect");
 }
@@ -412,7 +427,7 @@ Pbind(lua_State *L)
 	checknargs (L, 2);
 	fd = checkint(L, 1);
 	if (sockaddr_from_lua(L, 2, &sa, &salen) != 0)
-		return pusherror(L, "not a valid IPv4 dotted-decimal or IPv6 address string");
+		return pusherror(L, "not a valid IPv4 or IPv6 argument");
 
 	return pushresult(L, bind(fd, (struct sockaddr *)&sa, salen), "bind");
 }
@@ -602,7 +617,7 @@ Psendto(lua_State *L)
 	socklen_t salen;
 	checknargs (L, 3);
 	if (sockaddr_from_lua(L, 3, &sa, &salen) != 0)
-		return pusherror (L, "not a valid IPv4 dotted-decimal or IPv6 address string");
+		return pusherror (L, "not a valid IPv4 or IPv6 argument");
 
 	return pushresult(L, sendto(fd, buf, len, 0, (struct sockaddr *)&sa, salen), "sendto");
 }
