@@ -1,6 +1,6 @@
 /*
  * POSIX library for Lua 5.1, 5.2 & 5.3.
- * Copyright (C) 2013-2018 Gary V. Vaughan
+ * Copyright (C) 2013-2019 Gary V. Vaughan
  * Copyright (C) 2010-2013 Reuben Thomas <rrt@sc3d.org>
  * Copyright (C) 2008-2010 Natanael Copa <natanael.copa@gmail.com>
  * Clean up and bug fixes by Leo Razoumov <slonik.az@gmail.com> 2006-10-11
@@ -23,6 +23,8 @@
 #endif
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <grp.h>
+#include <pwd.h>
 #include <unistd.h>
 
 #include "_helpers.c"
@@ -321,7 +323,7 @@ runexec(lua_State *L, int use_shell)
 
 
 /***
-Execute a program without using the shell.
+Execute a program at exactly *path*..
 @function exec
 @string path
 @tparam table argt arguments (table can include index 0)
@@ -338,7 +340,7 @@ Pexec(lua_State *L)
 
 
 /***
-Execute a program using the shell.
+Execute a program found using command PATH search, like the shell.
 @function execp
 @string path
 @tparam table argt arguments (table can include index 0)
